@@ -36,7 +36,7 @@ static int output_buffer_rawN;
 gboolean test_mode;
 int last_input_method;
 #endif
-void set_wselkey(char *s);
+void set_wselkey();
 void gtab_set_win1_cb();
 void toggle_symbol_table();
 
@@ -71,17 +71,23 @@ void send_fake_key_eve(KeySym key)
 {
 #if WIN32
   win32_FakeKey(key, true);
+  Sleep(10);
   win32_FakeKey(key, false);
 #else
   KeyCode kc = XKeysymToKeycode(dpy, key);
   XTestFakeKeyEvent(dpy, kc, True, CurrentTime);
+  usleep(10000);
   XTestFakeKeyEvent(dpy, kc, False, CurrentTime);
 #endif
 }
 
 void fake_shift()
 {
+#if 0
+  send_fake_key_eve(XK_Control_L);
+#else
   send_fake_key_eve(XK_Shift_L);
+#endif
 }
 
 void swap_ptr(char **a, char **b)
@@ -94,7 +100,7 @@ void swap_ptr(char **a, char **b)
 int force_preedit=0;
 void force_preedit_shift()
 {
-  send_fake_key_eve(XK_Shift_L);
+  fake_shift();
   force_preedit=1;
 }
 
@@ -236,7 +242,8 @@ direct:
     if (pid) {
       close(pfdw[0]);
       close(pfdr[1]);
-      write(pfdw[1], text, len);
+      if (write(pfdw[1], text, len) < 0) {
+      }
       close(pfdw[1]);
       int rn = read(pfdr[0], filter_text, sizeof(filter_text) - 1);
       filter_text[rn] = 0;
@@ -356,7 +363,9 @@ int current_in_win_x = -1, current_in_win_y = -1;  // request x/y
 
 void reset_current_in_win_xy()
 {
+#if 0
   current_in_win_x = current_in_win_y = -1;
+#endif
 }
 
 GCIN_module_callback_functions *module_cb1(ClientState *cs)
@@ -499,11 +508,9 @@ void move_in_win(ClientState *cs, int x, int y)
     case method_type_PHO:
       move_win_pho(x, y);
       break;
-#if USE_TSIN
     case method_type_TSIN:
       move_win0(x, y);
       break;
-#endif
     case method_type_MODULE:
       if (inmd[cs->in_method].mod_cb_funcs)
         module_cb1(cs)->module_move_win(x, y);
@@ -542,6 +549,7 @@ void getRootXY(Window win, int wx, int wy, int *tx, int *ty)
 #endif
 }
 
+extern int dpy_x_ofs, dpy_y_ofs;
 
 void move_IC_in_win(ClientState *cs)
 {
@@ -578,6 +586,10 @@ void move_IC_in_win(ClientState *cs)
    if (inpwin) {
      RECT rect;
      GetClientRect((HWND)inpwin, &rect);
+
+	 rect.right -= dpy_x_ofs;
+	 rect.bottom -= dpy_y_ofs;
+
      if (inpx >= rect.right)
        inpx = rect.right - 1;
      if (inpy >= rect.bottom)
@@ -635,6 +647,8 @@ void update_in_win_pos()
   } else {
     move_IC_in_win(current_CS);
   }
+
+  disp_tray_icon();
 }
 
 void win_pho_disp_half_full();
@@ -650,10 +664,15 @@ extern void destroy_tray_icon();
 #if UNIX
 void destroy_tray()
 {
-  if (current_gcin_win32_icon)
+  if (current_gcin_win32_icon==GCIN_TRAY_WIN32)
     destroy_tray_win32();
   else
+  if (current_gcin_win32_icon==GCIN_TRAY_UNIX)
     destroy_tray_icon();
+#if USE_INDICATOR
+  else
+    destroy_tray_indicator();
+#endif
 }
 #endif
 
@@ -668,13 +687,17 @@ void disp_tray_icon()
 
   current_gcin_win32_icon = gcin_win32_icon;
 
-  if (gcin_win32_icon)
+  if (gcin_win32_icon==GCIN_TRAY_WIN32)
 #endif
-
     load_tray_icon_win32();
 #if UNIX
   else
+  if (gcin_win32_icon==GCIN_TRAY_UNIX)
     load_tray_icon();
+#if USE_INDICATOR
+  else
+    load_tray_icon_indicator();
+#endif
 #endif
 }
 #endif
@@ -703,16 +726,16 @@ void disp_im_half_full()
 
 void flush_tsin_buffer();
 void reset_gtab_all();
-void set_tsin_pho_mode0(ClientState *cs);
+void set_tsin_pho_mode0(ClientState *cs, gboolean tsin_pho_mode);
 
 //static u_int orig_caps_state;
 
-void init_state_chinese(ClientState *cs)
+void init_state_chinese(ClientState *cs, gboolean tsin_pho_mode)
 {
   dbg("init_state_chinese %p\n",cs);
 
   cs->im_state = GCIN_STATE_CHINESE;
-  set_tsin_pho_mode0(cs);
+  set_tsin_pho_mode0(cs, tsin_pho_mode);
   if (!cs->in_method)
 #if UNIX
     init_in_method2(cs, default_input_method);
@@ -727,7 +750,9 @@ void init_state_chinese(ClientState *cs)
 
 gboolean output_gbuf();
 void update_win_kbm();
-void flush_edit_buffer();
+void flush_edit_buffer(), ClrIn();
+void tsin_set_eng_ch(int nmod);
+int current_kbd_state;
 
 void toggle_im_enabled()
 {
@@ -737,6 +762,12 @@ void toggle_im_enabled()
     if (current_CS->in_method < 0)
       p_err("err found");
 
+    int status=0;
+
+//  dbg("feedkey_pp %x %x\n", xkey, kbstate);
+//  if (xkey=='1')
+//    dbg("aaa\n");
+
 
     if (current_CS->im_state != GCIN_STATE_DISABLED) {
       if (current_CS->im_state == GCIN_STATE_ENG_FULL) {
@@ -756,15 +787,14 @@ void toggle_im_enabled()
 
       update_win_kbm();
 
-#if TRAY_ENABLED
       disp_tray_icon();
-#endif
     } else {
       if (!current_method_type())
         init_gtab(current_CS->in_method);
 
+	  ClrIn();
 
-      init_state_chinese(current_CS);
+      init_state_chinese(current_CS, TRUE);
       reset_current_in_win_xy();
 #if 1
       show_in_win(current_CS);
@@ -776,9 +806,11 @@ void toggle_im_enabled()
 
       update_win_kbm();
 
-#if TRAY_ENABLED
+      if (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock) {
+        tsin_set_eng_ch(!(current_kbd_state&LockMask));
+      }
+
       disp_tray_icon();
-#endif
     }
 
     save_CS_current_to_temp();
@@ -840,25 +872,8 @@ gboolean win_is_visible()
 void disp_gtab_half_full(gboolean hf);
 void tsin_toggle_half_full();
 
-void toggle_half_full_char()
+void toggle_half_full_char_sub()
 {
-#if WIN32
-  if (test_mode)
-    return;
-#endif
-
-  check_CS();
-
-  if (!gcin_shift_space_eng_full) {
-    current_CS->b_half_full_char = 0;
-    tss.tsin_half_full=0;
-    disp_im_half_full();
-    return;
-  }
-
-
-//  dbg("toggle_half_full_char\n");
-
   if (current_method_type() == method_type_TSIN && current_CS->im_state == GCIN_STATE_CHINESE) {
     tsin_toggle_half_full();
   }
@@ -880,7 +895,26 @@ void toggle_half_full_char()
   }
 
   save_CS_current_to_temp();
-//  dbg("half full toggle\n");
+}
+
+void toggle_half_full_char()
+{
+#if WIN32
+  if (test_mode)
+    return;
+#endif
+
+  check_CS();
+
+  if (!gcin_shift_space_eng_full) {
+    current_CS->b_half_full_char = 0;
+    tss.tsin_half_full=0;
+    disp_im_half_full();
+    return;
+  }
+
+//  dbg("toggle_half_full_char\n");
+  toggle_half_full_char_sub() ;
 }
 
 void init_tab_pp(gboolean init);
@@ -898,8 +932,9 @@ GCIN_module_callback_functions *init_GCIN_module_callback_functions(char *sofile
 time_t find_tab_file(char *fname, char *out_file);
 gboolean tsin_page_up(), tsin_page_down();
 int tsin_sele_by_idx(int);
+void free_tsin();
 
-static void tsin_set_win1_cb()
+void tsin_set_win1_cb()
 {
   set_win1_cb((cb_selec_by_idx_t)tsin_sele_by_idx, (cb_page_ud_t)tsin_page_up, (cb_page_ud_t)tsin_page_down);
 }
@@ -912,7 +947,7 @@ void update_win_kbm_inited()
 
 gboolean init_in_method2(ClientState *cs, int in_no)
 {
-  dbg("init_in_method %p %d\n", cs, in_no);
+  dbg("init_in_method2 %p %d\n", cs, in_no);
   gboolean init_im = !(cur_inmd && (cur_inmd->flag & FLAG_GTAB_SYM_KBM));
 
   if (in_no < 0)
@@ -933,7 +968,7 @@ gboolean init_in_method2(ClientState *cs, int in_no)
     reset_current_in_win_xy();
 
 //  dbg("switch init_in_method %x %d\n", current_CS, in_no);
-  set_tsin_pho_mode0(cs);
+//  set_tsin_pho_mode0(cs, ini_tsin_pho_mode);
   tsin_set_win1_cb();
 
   switch (inmd[in_no].method_type) {
@@ -942,9 +977,11 @@ gboolean init_in_method2(ClientState *cs, int in_no)
       init_tab_pho();
       break;
     case method_type_TSIN:
-      set_wselkey(pho_selkey);
+	  free_tsin();
+      set_wselkey();
       cs->in_method = in_no;
-      init_tab_pp(init_im);
+      if (cs==current_CS || !current_CS)
+		init_tab_pp(init_im);
       break;
     case method_type_SYMBOL_TABLE:
       toggle_symbol_table();
@@ -964,11 +1001,11 @@ gboolean init_in_method2(ClientState *cs, int in_no)
         }
       }
 
-      if (inmd[in_no].mod_cb_funcs->module_init_win(&gmf)) {
+      if ((cs==current_CS || !current_CS) && inmd[in_no].mod_cb_funcs->module_init_win(&gmf)) {
         cs->in_method = in_no;
         module_cb()->module_show_win();
         if (cs==current_CS)
-          set_wselkey(pho_selkey);
+          set_wselkey();
       } else {
         return FALSE;
       }
@@ -984,7 +1021,10 @@ gboolean init_in_method2(ClientState *cs, int in_no)
     case method_type_GTAB:
     {
       dbg("method_type_GTAB\n");
-      init_gtab(in_no);
+	  free_tsin();
+      if (cs==current_CS || !current_CS)
+		init_gtab(in_no);
+
       if (!inmd[in_no].DefChars)
         return FALSE;
       cs->in_method = in_no;
@@ -1008,17 +1048,15 @@ gboolean init_in_method2(ClientState *cs, int in_no)
 #endif
 
   if (cs==current_CS) {
-#if TRAY_ENABLED
-    disp_tray_icon();
-#endif
-
-    if (inmd[current_CS->in_method].selkey) {
-      set_wselkey(inmd[current_CS->in_method].selkey);
+	char *selkey;
+    if (selkey=inmd[current_CS->in_method].selkey) {
+      set_wselkey();
       gtab_set_win1_cb();
   //    dbg("aa selkey %s\n", inmd[current_CS->in_method].selkey);
     }
     update_in_win_pos();
     update_win_kbm_inited();
+    disp_tray_icon();
   }
 
   return TRUE;
@@ -1084,7 +1122,7 @@ gboolean full_char_proc(KeySym keysym)
 }
 
 int feedkey_pho(KeySym xkey, int kbstate);
-int feedkey_pp(KeySym xkey, int state);
+int feedkey_tsin(KeySym xkey, int state);
 int feedkey_gtab(KeySym key, int kbstate);
 int feed_phrase(KeySym ksym, int state);
 void tsin_set_eng_ch(int nmod);
@@ -1101,7 +1139,8 @@ gboolean timeout_raise_window(gpointer data)
 extern Window xwin_pho, xwin0, xwin_gtab;
 void create_win_sym(), win_kbm_disp_caplock();
 
-#if !GTK_CHECK_VERSION(2,16,0)
+#if !GTK_CHECK_VERSION(2,16,0) && 0
+#include <X11/extensions/XKBsrv.h>
 gboolean get_caps_lock_state()
 {
   XkbStateRec states;
@@ -1113,13 +1152,15 @@ gboolean get_caps_lock_state()
 }
 #endif
 
+gboolean capslock_on;
+
 void disp_win_kbm_capslock()
 {
   if (!b_show_win_kbm)
     return;
 
   gboolean o_state = old_capslock_on;
-  old_capslock_on = gdk_keymap_get_caps_lock_state(gdk_keymap_get_default());
+  old_capslock_on = capslock_on;
 
 //  dbg("%x %x\n", old_capslock_on, o_state);
 
@@ -1161,6 +1202,8 @@ void toggle_symbol_table()
 void destroy_phrase_save_menu();
 int gcin_switch_keys_lookup(int key);
 gboolean b_menu_key_pressed;
+void cb_trad_sim_toggle();
+void set_tsin_pho_mode();
 
 // return TRUE if the key press is processed
 gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
@@ -1168,8 +1211,18 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 #if 0
   dbg("key press %x %x\n", keysym, kev_state);
 #endif
+
+  current_kbd_state = kev_state;
   destroy_phrase_save_menu();
 
+  capslock_on = (kev_state&LockMask) > 0;
+  if (keysym == XK_Caps_Lock) {
+#if UNIX
+    capslock_on = !(kev_state&LockMask);
+#endif
+    disp_tray_icon();
+  }
+
   disp_win_kbm_capslock();
   check_CS();
 
@@ -1186,8 +1239,9 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
     return TRUE;
   }
 
-  if (force_preedit) {
-    force_preedit=0;
+  if (force_preedit==1) {
+	dbg("orce_preedit==1\n");
+    force_preedit = 2;
     return 1;
   }
 
@@ -1215,7 +1269,7 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
       ((kev_state & ShiftMask) && gcin_im_toggle_keys==Shift_Space) ||
       ((kev_state & Mod4Mask) && gcin_im_toggle_keys==Windows_Space)
     ) {
-      if (current_method_type() == method_type_TSIN) {
+      if (current_method_type() == method_type_TSIN && tsin_chinese_english_toggle_key != TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock) {
         tsin_set_eng_ch(1);
       }
 
@@ -1238,7 +1292,8 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
   }
 
 
-  if ((kev_state & (Mod1Mask|ShiftMask)) == (Mod1Mask|ShiftMask)) {
+  if (((kev_state & (Mod1Mask|ShiftMask)) == (Mod1Mask|ShiftMask)) ||
+	  (gcin_ctrl_punc && (kev_state&(Mod1Mask|Mod5Mask|ShiftMask|ControlMask))==ControlMask && keysym < 127 && !(keysym>='0' && keysym<='9')) ) {
     if (current_CS->im_state != GCIN_STATE_DISABLED || gcin_eng_phrase_enabled)
       return feed_phrase(keysym, kev_state);
     else
@@ -1251,21 +1306,43 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
   }
 
 
-  if ((kev_state & ControlMask) && (kev_state&(Mod1Mask|Mod5Mask))) {
-    if (keysym == 'g' || keysym == 'r') {
+  if ((kev_state & ControlMask) && (kev_state&(Mod1Mask|Mod5Mask|ShiftMask))) {
+    if (keysym == 'g' || keysym == 'r' || keysym == XK_F5) {
+      last_keysym = 0;
       send_output_buffer_bak();
       return TRUE;
     }
 
+    if (keysym == XK_F8) {
+      last_keysym = 0;
+      cb_trad_sim_toggle();
+    }
+
     if (!gcin_enable_ctrl_alt_switch)
       return FALSE;
 
+#if WIN32
+	if (kev_state & ShiftMask) {
+		struct {
+			KeySym frm, to;
+		} tkeys[]={{0x1e, '6'}};
+
+		for(int i=0; tkeys[i].frm; i++) {
+			if (tkeys[i].frm == keysym) {
+				keysym = tkeys[i].to;
+				break;
+			}
+		}
+	}
+#endif
+
     int kidx = gcin_switch_keys_lookup(keysym);
-    if (kidx < 0)
+    last_keysym = 0;
+    if (kidx < 0) {
       return FALSE;
+    }
 
     if (inmd[kidx].method_type == method_type_SYMBOL_TABLE) {
-
       toggle_symbol_table();
       return TRUE;
     }
@@ -1277,7 +1354,10 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 #if WIN32
     if (!test_mode)
 #endif
+    {
       init_in_method(kidx);
+      set_tsin_pho_mode();
+    }
 
     return TRUE;
   }
@@ -1288,6 +1368,7 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
     return FALSE;
   }
 
+#if 0
   if (!current_CS->b_gcin_protocol) {
   if (((keysym == XK_Control_L || keysym == XK_Control_R)
                    && (kev_state & ShiftMask)) ||
@@ -1297,6 +1378,7 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
      return TRUE;
   }
   }
+#endif
 
   if (current_CS->b_raise_window && keysym>=' ' && keysym < 127) {
     if (timeout_handle)
@@ -1304,18 +1386,12 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
     timeout_handle = g_timeout_add(200, timeout_raise_window, NULL);
   }
 
-  if (kev_state & ControlMask) {
-    if (feed_phrase(keysym, kev_state))
-      return TRUE;
-  }
 
   switch(current_method_type()) {
     case method_type_PHO:
       return feedkey_pho(keysym, kev_state);
-#if USE_TSIN
     case method_type_TSIN:
-      return feedkey_pp(keysym, kev_state);
-#endif
+      return feedkey_tsin(keysym, kev_state);
     case method_type_MODULE:
       if (!module_cb())
         return FALSE;
@@ -1335,9 +1411,18 @@ int feedkey_gtab_release(KeySym xkey, int kbstate);
 // return TRUE if the key press is processed
 gboolean ProcessKeyRelease(KeySym keysym, u_int kev_state)
 {
+  check_CS();
+  current_kbd_state = kev_state;
+
+  if (force_preedit==2) {
+	dbg("orce_preedit==2\n");
+	force_preedit = 0;
+	return TRUE;
+  }
+
   disp_win_kbm_capslock();
 
-  check_CS();
+//  check_CS();
 #if 0
   dbg_time("key release %x %x\n", keysym, kev_state);
 #endif
@@ -1345,7 +1430,10 @@ gboolean ProcessKeyRelease(KeySym keysym, u_int kev_state)
   if (current_CS->im_state == GCIN_STATE_DISABLED)
     return FALSE;
 
+
 #if 1
+//  dbg("last_keysym %x\n", last_keysym);
+
   if (current_CS->b_gcin_protocol && (last_keysym == XK_Shift_L ||
   last_keysym == XK_Shift_R || last_keysym == XK_Control_L || last_keysym == XK_Control_R)) {
     if (((keysym == XK_Control_L || keysym == XK_Control_R)
@@ -1407,31 +1495,50 @@ int xim_ForwardEventHandler(IMForwardEventStruct *call_data)
 }
 #endif
 
-#if UNIX
-int skip_window(Window win)
-{
-  XWindowAttributes att;
-  XGetWindowAttributes(dpy, win, &att);
-#if 0
-  dbg("hhh %d %d class:%d all:%x %x\n", att.width, att.height, att.class,
-    att.all_event_masks, att.your_event_mask);
-#endif
-  if (att.override_redirect)
-    return 1;
-
-  return 0;
-}
-#endif
 
 void gcin_reset();
 
+#if UNIX
+gboolean is_tip_window(Window inpwin)
+{
+   // Dirty fix for chrome, doesn't work well.
+   if (!inpwin)
+     return FALSE;
+   XWindowAttributes att;
+   XGetWindowAttributes(dpy, inpwin, &att);
+
+   dbg("%d, %d\n", att.width, att.height);
+// chrome window is override_redirect
+//   if (att.override_redirect)
+//     return;
+
+   return att.override_redirect && att.height < 24;
+}
+#endif
+
 int gcin_FocusIn(ClientState *cs)
 {
-  dbg("gcin_FocusIn %x\n", cs);
-  Window win = cs->client_win;
-  if (!win)
+  dbg("gcin_FocusIn %p\n", cs);
+  if (!cs) {
+    dbg("cs is null");
     return FALSE;
+  } else {
+//    dbg("gcin_FocusIn %x\n", cs);
+  }
 
+  Window win = cs->client_win;
+  dbg("win: %x\n", win);
+
+  if (!win) {
+	dbg("!win");
+    return FALSE;
+  }
+#if UNIX && 0
+  if (is_tip_window(win)) {
+	dbg("is_tip_window\n");
+	return FALSE;
+  }
+#endif
   reset_current_in_win_xy();
 
   if (cs) {
@@ -1439,6 +1546,7 @@ int gcin_FocusIn(ClientState *cs)
 
     if (focus_win != win) {
 #if 1
+      dbg("reset %x,%x\n", focus_win, win);
       gcin_reset();
 #endif
       hide_in_win(current_CS);
@@ -1455,22 +1563,29 @@ int gcin_FocusIn(ClientState *cs)
     if (cs->im_state != GCIN_STATE_DISABLED) {
       show_in_win(cs);
       move_IC_in_win(cs);
-    } else
+    } else {
       hide_in_win(cs);
+      move_IC_in_win(cs);
+    }
   }
 
-  if (inmd[cs->in_method].selkey)
-    set_wselkey(inmd[cs->in_method].selkey);
-  else {
-    set_wselkey(pho_selkey);
+  cur_inmd = &inmd[current_CS->in_method];
+
+//  if (inmd[cs->in_method].selkey)
+//    set_wselkey();
+//  else
+  {
+    set_wselkey();
+#if 0
     gtab_set_win1_cb();
     tsin_set_win1_cb();
+#endif
   }
 
   update_win_kbm();
-#if TRAY_ENABLED
+
   disp_tray_icon();
-#endif
+
 #if 0
   dbg_time("gcin_FocusIn %x %x\n",cs, current_CS);
 #endif
@@ -1514,9 +1629,21 @@ int gcin_FocusOut(ClientState *cs)
   if (cs != current_CS)
      return FALSE;
 
+  if (!cs) {
+	 dbg("gcin_FocusOut is null\n");
+	 return FALSE;
+  }
+
   if (!cs->client_win)
     return FALSE;
 
+#if UNIX && 0
+  if (is_tip_window(cs->client_win)) {
+	return FALSE;
+  }
+#endif
+
+
   if (t - last_focus_out_time < 100000) {
     last_focus_out_time = t;
     return FALSE;
@@ -1588,7 +1715,7 @@ void gcin_reset()
 #if 1
   if (!current_CS)
     return;
-  dbg("gcin_reset\n");
+//  dbg("gcin_reset\n");
 
   switch(current_method_type()) {
     case method_type_PHO:
@@ -1610,6 +1737,19 @@ void gcin_reset()
 #endif
 }
 
+#if WIN32
+void show_tsin_stat();
+void gcin_set_tsin_pho_mode(ClientState *cs, gboolean pho_mode)
+{
+	set_tsin_pho_mode0(cs, pho_mode);
+
+	if (!pho_mode) {
+		cs->b_half_full_char = 0;
+		tss.tsin_half_full = 0;
+	}
+	show_tsin_stat();
+}
+#endif
 
 #if USE_XIM
 int xim_gcin_FocusOut(IMChangeFocusStruct *call_data)
