@@ -37,7 +37,11 @@ static GtkWidget *opt_im_toggle_keys, *check_button_gcin_remote_client,
        *check_button_gcin_init_im_enabled,
        *check_button_gcin_eng_phrase_enabled,
        *check_button_gcin_win_sym_click_close,
-       *check_button_gcin_punc_auto_send;
+       *check_button_gcin_punc_auto_send,
+       *check_button_gcin_ctrl_punc,
+       *check_button_ini_tsin_pho_mode,
+       *check_button_gcin_escape_clear_edit_buffer,
+       *check_button_gcin_buffer_select_char_auto_right, *check_button_en_pre_select;
 #if USE_GCB
 static GtkWidget *spinner_gcb_position_x, *spinner_gcb_position_y;
 static GtkWidget *spinner_gcb_history_n, *spinner_gcb_button_n;
@@ -205,7 +209,10 @@ static void save_gtab_list()
     char *icon = pinmd->icon;
     char *disabled = pinmd->disabled?"!":"";
 
-    fprintf(fp, "%s%s %c %s %s\n", disabled,name, pinmd->key_ch, file, icon);
+    if (pinmd->phrase_txt)
+      fprintf(fp, "%s%s %c %s %s %s\n", disabled,name, pinmd->key_ch, file, icon, pinmd->phrase_txt);
+    else
+      fprintf(fp, "%s%s %c %s %s\n", disabled,name, pinmd->key_ch, file, icon);
   }
 
   fclose(fp);
@@ -268,6 +275,9 @@ static void cb_ok (GtkWidget *button, gpointer data)
   save_gcin_conf_int(GCIN_INIT_IM_ENABLED,
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_init_im_enabled)));
 
+  save_gcin_conf_int(TSIN_PHO_MODE,
+    !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_ini_tsin_pho_mode)));
+
   save_gcin_conf_int(GCIN_ENG_PHRASE_ENABLED,
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_eng_phrase_enabled)));
 
@@ -279,6 +289,19 @@ static void cb_ok (GtkWidget *button, gpointer data)
 
   save_gcin_conf_int(GCIN_PUNC_AUTO_SEND,
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_punc_auto_send)));
+
+  save_gcin_conf_int(GCIN_ESCAPE_CLEAR_EDIT_BUFFER,
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_escape_clear_edit_buffer)));
+
+  save_gcin_conf_int(GCIN_CTRL_PUNC,
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_ctrl_punc)));
+
+  save_gcin_conf_int(GCIN_BUFFER_SELECT_CHAR_AUTO_RIGHT,
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_buffer_select_char_auto_right)));
+
+  save_gcin_conf_int(EN_PRE_SELECT,
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_en_pre_select)));
+
 #if UNIX
   save_gcin_conf_int(GCIN_SINGLE_STATE,
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_single_state)));
@@ -315,6 +338,8 @@ static void cb_ok (GtkWidget *button, gpointer data)
 	  GDK_DISPLAY(),
 #endif
 	  "reload");
+
+   load_setttings();
 }
 
 static void cb_cancel (GtkWidget *widget, gpointer data)
@@ -452,7 +477,13 @@ add_columns (GtkTreeView *treeview)
   g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_KEY);
 
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
-                                               -1, _(_L("Ctrl-Alt-鍵")), renderer,
+                                               -1,
+#if UNIX
+											   _(_L("Ctrl-Alt-鍵"))
+#else
+											   _(_L("Ctrl-Shift-鍵"))
+#endif
+											   , renderer,
                                                "text", COLUMN_KEY,
                                                "editable", COLUMN_EDITABLE,
                                                NULL);
@@ -748,6 +779,16 @@ void create_gtablist_window (void)
 #endif
 
 
+  GtkWidget *hbox_ini_tsin_pho_mode = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX(vboxR), hbox_ini_tsin_pho_mode, FALSE, FALSE, 0);
+  GtkWidget *label_ini_tsin_pho_mode = gtk_label_new(_(_L("Shift切換中英初始英數")));
+  gtk_box_pack_start (GTK_BOX (hbox_ini_tsin_pho_mode), label_ini_tsin_pho_mode,  FALSE, FALSE, 0);
+  check_button_ini_tsin_pho_mode = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_ini_tsin_pho_mode),check_button_ini_tsin_pho_mode,  FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_ini_tsin_pho_mode),
+     !ini_tsin_pho_mode);
+
+
   GtkWidget *hbox_gcin_shift_space_eng_full = gtk_hbox_new (FALSE, 10);
   gtk_box_pack_start (GTK_BOX (vboxR), hbox_gcin_shift_space_eng_full, FALSE, FALSE, 0);
   GtkWidget *label_gcin_shift_space_eng_full = gtk_label_new(_(_L("shift-space 進入全形英文模式")));
@@ -770,7 +811,7 @@ void create_gtablist_window (void)
 
   GtkWidget *hbox_gcin_eng_phrase_enabled = gtk_hbox_new (FALSE, 10);
   gtk_box_pack_start (GTK_BOX (vboxR), hbox_gcin_eng_phrase_enabled, FALSE, FALSE, 0);
-  GtkWidget *label_gcin_eng_phrase_enabled = gtk_label_new(_(_L("英數狀態使用 alt-shift 片語")));
+  GtkWidget *label_gcin_eng_phrase_enabled = gtk_label_new(_(_L("英數狀態使用 alt-shift/Ctrl 片語")));
   gtk_box_pack_start (GTK_BOX (hbox_gcin_eng_phrase_enabled), label_gcin_eng_phrase_enabled,  FALSE, FALSE, 0);
   check_button_gcin_eng_phrase_enabled = gtk_check_button_new ();
   gtk_box_pack_start (GTK_BOX (hbox_gcin_eng_phrase_enabled),check_button_gcin_eng_phrase_enabled,  FALSE, FALSE, 0);
@@ -806,6 +847,14 @@ void create_gtablist_window (void)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_bell_off),
      gcin_bell_off);
 
+  GtkWidget *hbox_gcin_ctrl_punc = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX (vboxR), hbox_gcin_ctrl_punc, FALSE, FALSE, 0);
+  GtkWidget *label_gcin_ctrl_punc = gtk_label_new(_(_L("Ctrl 輸入標點符號")));
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_ctrl_punc), label_gcin_ctrl_punc,  FALSE, FALSE, 0);
+  check_button_gcin_ctrl_punc = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_ctrl_punc),check_button_gcin_ctrl_punc,  FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_ctrl_punc),
+     gcin_ctrl_punc);
 
   GtkWidget *hbox_gcin_punc_auto_send = gtk_hbox_new (FALSE, 10);
   gtk_box_pack_start (GTK_BOX (vboxL), hbox_gcin_punc_auto_send, FALSE, FALSE, 0);
@@ -816,6 +865,32 @@ void create_gtablist_window (void)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_punc_auto_send),
      gcin_punc_auto_send);
 
+  GtkWidget *hbox_gcin_escape_clear_edit_buffer = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX (vboxL), hbox_gcin_escape_clear_edit_buffer, FALSE, FALSE, 0);
+  GtkWidget *label_gcin_escape_clear_edit_buffer = gtk_label_new(_(_L("Esc清除編輯區內容")));
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_escape_clear_edit_buffer), label_gcin_escape_clear_edit_buffer,  FALSE, FALSE, 0);
+  check_button_gcin_escape_clear_edit_buffer = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_escape_clear_edit_buffer),check_button_gcin_escape_clear_edit_buffer,  FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_escape_clear_edit_buffer),
+     gcin_escape_clear_edit_buffer);
+
+  GtkWidget *hbox_gcin_buffer_select_char_auto_left = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX (vboxL), hbox_gcin_buffer_select_char_auto_left, FALSE, FALSE, 0);
+  GtkWidget *label_gcin_buffer_select_char_auto_left = gtk_label_new(_(_L("選字後游標自動往右移")));
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_buffer_select_char_auto_left), label_gcin_buffer_select_char_auto_left,  FALSE, FALSE, 0);
+  check_button_gcin_buffer_select_char_auto_right = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_buffer_select_char_auto_left),check_button_gcin_buffer_select_char_auto_right,  FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_buffer_select_char_auto_right),
+     gcin_buffer_select_char_auto_right);
+
+  GtkWidget *hbox_en_pre_select = gtk_hbox_new (FALSE, 10);
+  gtk_box_pack_start (GTK_BOX (vboxR), hbox_en_pre_select, FALSE, FALSE, 0);
+  GtkWidget *label_en_pre_select = gtk_label_new(_(_L("編輯區英數預選詞")));
+  gtk_box_pack_start (GTK_BOX (hbox_en_pre_select), label_en_pre_select,  FALSE, FALSE, 0);
+  check_button_en_pre_select = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_en_pre_select),check_button_en_pre_select,  FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_en_pre_select), en_pre_select);
+
 
 #if USE_GCB
   GtkWidget *hbox_gcb_pos = gtk_hbox_new (FALSE, 10);
