@@ -21,12 +21,13 @@ static GtkWidget *check_button_tsin_phrase_pre_select,
                  *check_button_tsin_tail_select_key,
                  *check_button_tsin_buffer_editing_mode,
                  *check_button_tsin_use_pho_near,
+                 *check_button_tsin_shift_punc,
                  *spinner_tsin_buffer_size,
                  *spinner_pho_candicate_col_N;
 
 GtkWidget *check_button_gcin_capslock_lower;
 
-static GtkWidget *opt_kbm_opts, *opt_selkeys, *opt_eng_ch_opts;
+static GtkWidget *opt_kbm_opts, *opt_selkeys, *opt_eng_ch_opts, *opt_tsin_space;
 extern gboolean button_order;
 
 
@@ -35,7 +36,6 @@ static struct {
   int key;
 } tsin_eng_ch_sw[]={
   {N_(_L("CapsLock")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock},
-//  {N_(_L("Tab")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Tab},
   {N_(_L("Shift")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift},
   {N_(_L("左Shift")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL},
   {N_(_L("右Shift")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR},
@@ -47,8 +47,9 @@ static struct {
   unich_t *name;
   int key;
 } tsin_space_options[]={
-  {N_(_L("選擇同音字")), TSIN_SPACE_OPT_SELECT_CHAR},
-  {N_(_L("輸入空白")), TSIN_SPACE_OPT_INPUT}
+  {N_(_L("選擇字詞")), TSIN_SPACE_OPT_SELECT_CHAR},
+  {N_(_L("輸入空白")), TSIN_SPACE_OPT_INPUT},
+  {N_(_L("送出編輯區內容")), TSIN_SPACE_OPT_FLUSH_EDIT}
 };
 int tsin_space_optionsN = sizeof(tsin_space_options) / sizeof(tsin_space_options[0]);
 
@@ -83,6 +84,12 @@ static GtkWidget *gcin_kbm_window = NULL;
 static int new_select_idx_tsin_space_opt;
 static GdkColor tsin_phrase_line_gcolor;
 
+void save_tsin_space_opt()
+{
+  save_gcin_conf_int(TSIN_SPACE_OPT,
+                     tsin_space_options[gtk_combo_box_get_active(GTK_COMBO_BOX(opt_tsin_space))].key);
+}
+
 
 static gboolean cb_ok( GtkWidget *widget,
                                    GdkEvent  *event,
@@ -111,9 +118,8 @@ static gboolean cb_ok( GtkWidget *widget,
   save_gcin_conf_str(PHONETIC_KEYBOARD, tt);
 
   save_tsin_eng_pho_key();
-
-  save_gcin_conf_int(TSIN_SPACE_OPT,
-                     tsin_space_options[new_select_idx_tsin_space_opt].key);
+                     
+  save_tsin_space_opt();
 
   save_gcin_conf_int(TSIN_PHRASE_PRE_SELECT,
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_phrase_pre_select)));
@@ -146,6 +152,9 @@ static gboolean cb_ok( GtkWidget *widget,
   save_gcin_conf_int(TSIN_BUFFER_EDITING_MODE,
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_buffer_editing_mode)));
 
+ save_gcin_conf_int(TSIN_SHIFT_PUNC,
+       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_shift_punc)));
+
   tsin_buffer_size = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_tsin_buffer_size));
   save_gcin_conf_int(TSIN_BUFFER_SIZE, tsin_buffer_size);
 
@@ -200,17 +209,6 @@ static int get_currnet_eng_ch_sw_idx()
 }
 
 
-static int get_currnet_tsin_space_option_idx()
-{
-  int i;
-  for(i=0; i < tsin_space_optionsN; i++)
-    if (tsin_space_options[i].key == tsin_space_opt)
-      return i;
-
-  p_err("tsin-space-opt->%d is not valid", tsin_space_opt);
-  return -1;
-}
-
 static gboolean close_kbm_window( GtkWidget *widget,
                                    GdkEvent  *event,
                                    gpointer   data )
@@ -368,6 +366,43 @@ static GtkWidget *create_eng_ch_opts()
   return hbox;
 }
 
+
+static int get_currnet_tsin_space_option_idx()
+{
+  int i;
+  for(i=0; i < tsin_space_optionsN; i++)
+    if (tsin_space_options[i].key == tsin_space_opt)
+      return i;
+
+  p_err("tsin-space-opt->%d is not valid", tsin_space_opt);
+  return -1;
+}
+
+GtkWidget *create_tsin_space_opts()
+{
+  GtkWidget *hbox = gtk_hbox_new (FALSE, 1);
+
+  GtkWidget *label_tsin_space_opt = gtk_label_new(_(_L("編輯區空白鍵選項")));
+  gtk_box_pack_start (GTK_BOX (hbox), label_tsin_space_opt, FALSE, FALSE, 0);
+
+  opt_tsin_space = gtk_combo_box_new_text ();
+  gtk_box_pack_start (GTK_BOX (hbox), opt_tsin_space, FALSE, FALSE, 0);
+
+  int i;
+  int current_idx = get_currnet_tsin_space_option_idx();
+
+  for(i=0; i < tsin_space_optionsN; i++) {
+    gtk_combo_box_append_text (GTK_COMBO_BOX_TEXT (opt_tsin_space), _(tsin_space_options[i].name));
+  }
+
+  dbg("current_idx:%d\n", current_idx);
+
+  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_tsin_space), current_idx);
+
+  return hbox;
+}
+
+
 GtkWidget *create_en_pho_key_sel(char *s)
 {
 GtkWidget *frame_tsin_sw = gtk_frame_new(s);
@@ -408,7 +443,7 @@ void create_kbm_window()
                     G_CALLBACK (close_kbm_window),
                     NULL);
 
-  gtk_window_set_title (GTK_WINDOW (gcin_kbm_window), _(_L("gcin 注音/詞音設定")));
+  gtk_window_set_title (GTK_WINDOW (gcin_kbm_window), _(_L("gcin 注音/詞音/拼音設定")));
   gtk_container_set_border_width (GTK_CONTAINER (gcin_kbm_window), 1);
 
   GtkWidget *vbox_top = gtk_vbox_new (FALSE, 3);
@@ -433,31 +468,11 @@ void create_kbm_window()
 
   gtk_box_pack_start (GTK_BOX (vbox_l), create_en_pho_key_sel(_(_L("詞音輸入[中/英]切換"))), TRUE, TRUE, 0);
 
-  GtkWidget *frame_tsin_space_opt = gtk_frame_new(_(_L("詞音輸入空白鍵選項")));
-  gtk_box_pack_start (GTK_BOX (vbox_l), frame_tsin_space_opt, TRUE, TRUE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (frame_tsin_space_opt), 1);
-
-  GtkWidget *box_tsin_space_opt = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (frame_tsin_space_opt), box_tsin_space_opt);
-  gtk_container_set_border_width (GTK_CONTAINER (box_tsin_space_opt), 1);
-
-  GSList *group_tsin_space_opt = NULL;
-  int current_idx = get_currnet_tsin_space_option_idx();
-  new_select_idx_tsin_space_opt = current_idx;
-
-  gsize i;
-  for(i=0; i< tsin_space_optionsN; i++) {
-    GtkWidget *button = gtk_radio_button_new_with_label (group_tsin_space_opt, _(tsin_space_options[i].name));
-    gtk_box_pack_start (GTK_BOX (box_tsin_space_opt), button, TRUE, TRUE, 0);
-
-    group_tsin_space_opt = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-
-    g_signal_connect (G_OBJECT (button), "clicked",
-       G_CALLBACK (callback_button_clicked_tsin_space_opt), (gpointer) i);
-
-    if (i==current_idx)
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-  }
+  
+  gtk_box_pack_start (GTK_BOX (vbox_l), create_tsin_space_opts(), FALSE, FALSE, 0);
+//  gtk_container_set_border_width (GTK_CONTAINER (frame_tsin_space_opt), 1); 
+//  gtk_container_add (GTK_CONTAINER (frame_tsin_space_opt), );
+ 
 
   GtkWidget *hbox_tsin_phrase_pre_select = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox_l), hbox_tsin_phrase_pre_select , TRUE, TRUE, 1);
@@ -575,6 +590,17 @@ void create_kbm_window()
   gtk_toggle_button_set_active(
      GTK_TOGGLE_BUTTON(check_button_tsin_use_pho_near), tsin_use_pho_near);
 
+  GtkWidget *hbox_tsin_shift_punc = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox_r), hbox_tsin_shift_punc , TRUE, TRUE, 1);
+  GtkWidget *label_tsin_shift_punc = gtk_label_new(_(_L("Shift 輸入標點符號")));
+  gtk_widget_set_hexpand (label_tsin_shift_punc, TRUE);
+  gtk_widget_set_halign (label_tsin_shift_punc, GTK_ALIGN_CENTER);
+  gtk_box_pack_start (GTK_BOX (hbox_tsin_shift_punc), label_tsin_shift_punc , TRUE, TRUE, 0);
+  check_button_tsin_shift_punc = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_tsin_shift_punc), check_button_tsin_shift_punc, FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(
+     GTK_TOGGLE_BUTTON(check_button_tsin_shift_punc), tsin_shift_punc);
+
   GtkWidget *frame_tsin_buffer_size = gtk_frame_new(_(_L("詞音編輯緩衝區大小")));
   gtk_box_pack_start (GTK_BOX (vbox_r), frame_tsin_buffer_size, FALSE, FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (frame_tsin_buffer_size), 1);
