@@ -41,6 +41,7 @@ void load_gtab_list(gboolean skip_disabled)
     free(pinmd->filename); pinmd->filename=NULL;
     free(pinmd->cname); pinmd->cname=NULL;
     free(pinmd->icon); pinmd->icon=NULL;
+    free(pinmd->phrase_txt); pinmd->phrase_txt=NULL;
   }
 
   inmdN = 0;
@@ -56,6 +57,7 @@ void load_gtab_list(gboolean skip_disabled)
     char key[32];
     char file[32];
     char icon[128];
+    char phrase_txt[128];
 
     inmd = trealloc(inmd, INMD, inmdN);
 
@@ -63,6 +65,7 @@ void load_gtab_list(gboolean skip_disabled)
     key[0]=0;
     file[0]=0;
     icon[0]=0;
+    phrase_txt[0]=0;
 
     line[0]=0;
     myfgets(line, sizeof(line), fp);
@@ -77,7 +80,7 @@ void load_gtab_list(gboolean skip_disabled)
       continue;
 
 
-    sscanf(line, "%s %s %s %s", name, key, file, icon);
+    sscanf(line, "%s %s %s %s %s", name, key, file, icon, phrase_txt);
 //    dbg("%s %c\n", line, key[0]);
 
     if (strlen(name) < 1)
@@ -145,8 +148,11 @@ void load_gtab_list(gboolean skip_disabled)
 
     pinmd->cname = strdup(name);
 
-    if (strlen(icon))
+    if (icon[0])
       pinmd->icon = strdup(icon);
+
+    if (phrase_txt[0] && phrase_txt[0]!='-')
+      pinmd->phrase_txt = strdup(phrase_txt);
   }
   fclose(fp);
 
