@@ -11,11 +11,11 @@ typedef u_int KeySym;
 typedef unsigned char u_char;
 typedef unsigned int CARD32;
 typedef __int64 u_int64_t;
-void win32exec(char *s);
+bool win32exec(char *s);
 int win32exec_script(char *s, char *para=NULL);
 int win32exec_script_va(char *s, ...);
 int win32exec_va(char *s, ...);
-void win32exec_para(char *s, char *para);
+bool win32exec_para(char *s, char *para);
 extern char *gcin_program_files_path;
 extern char *gcin_script_path;
 typedef struct {
