@@ -110,8 +110,12 @@ static void create_char(int index)
 {
   int i;
 
-  if (!hbox_edit)
+//  dbg("create_char %d\n", index);
+
+  if (!hbox_edit) {
+	dbg("create_char: !hbox_edit\n");
     return;
+  }
 
   GdkColor fg;
   gdk_color_parse(gcin_win_color_fg, &fg);
@@ -157,13 +161,19 @@ extern gboolean b_use_full_space;
 
 void set_label_space();
 void set_label_space(GtkWidget *label);
+void create_win0();
 
 void disp_char(int index, char *ch)
 {
   if (gcin_edit_display_ap_only())
     return;
-  if (!top_bin)
-    return;
+
+  if (!top_bin) {
+//	dbg("not top_bin");
+	create_win0();
+	create_win0_gui();
+//    return;
+  }
 
 //  dbg("disp_char %d %s\n", index, ch);
   create_char(index);
@@ -265,8 +275,16 @@ void disp_tsin_pho(int index, char *pho)
 }
 
 void disp_tsin_pho(int index, char *pho);
+gboolean tsin_pho_mode();
+void show_button_pho(gboolean bshow);
+
 void clr_in_area_pho_tsin()
 {
+  if (!tsin_pho_mode()) {
+    show_button_pho(FALSE);
+    return;
+  }
+
   int i;
 #if WIN32
   if (test_mode)
@@ -460,6 +478,27 @@ static void mouse_button_callback( GtkWidget *widget,GdkEventButton *event, gpoi
 void tsin_toggle_eng_ch();
 void set_no_focus();
 
+GtkWidget *create_no_focus_win()
+{
+  GtkWidget *win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+//  gtk_window_set_has_resize_grip(GTK_WINDOW(win), FALSE);
+#if UNIX
+  gtk_window_set_resizable(GTK_WINDOW(win), FALSE);
+#endif
+#if WIN32
+  set_no_focus(win);
+#endif
+  gtk_container_set_border_width (GTK_CONTAINER (win), 0);
+  gtk_widget_realize (win);
+#if UNIX
+//  GdkWindow *gdkwin = gtk_widget_get_window(win);
+  set_no_focus(win);
+#else
+  win32_init_win(win);
+#endif
+
+  return win;
+}
 
 void create_win0()
 {
@@ -468,22 +507,7 @@ void create_win0()
 #if _DEBUG && 0
   dbg("create_win0\n");
 #endif
-  gwin0 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_has_resize_grip(GTK_WINDOW(gwin0), FALSE);
-#if UNIX
-  gtk_window_set_resizable(GTK_WINDOW(gwin0), FALSE);
-#endif
-#if WIN32
-  set_no_focus(gwin0);
-#endif
-  gtk_container_set_border_width (GTK_CONTAINER (gwin0), 0);
-  gtk_widget_realize (gwin0);
-#if UNIX
-  GdkWindow *gdkwin0 = gtk_widget_get_window(gwin0);
-  set_no_focus(gwin0);
-#else
-  win32_init_win(gwin0);
-#endif
+  gwin0 = create_no_focus_win();
 }
 
 
@@ -770,10 +794,14 @@ void win_tsin_disp_half_full()
   if (test_mode)
     return;
 #endif
-  if (gcin_win_color_use)
-   gtk_label_set_markup(GTK_LABEL(label_pho), get_full_str());
-  else
-    gtk_label_set_text(GTK_LABEL(label_pho), get_full_str());
+
+  if (label_pho) {
+    if (gcin_win_color_use)
+     gtk_label_set_markup(GTK_LABEL(label_pho), get_full_str());
+    else
+      gtk_label_set_text(GTK_LABEL(label_pho), get_full_str());
+  }
+
   compact_win0();
 }
 
