@@ -2,15 +2,16 @@
 #include "pho.h"
 #include "gst.h"
 #include "win1.h"
+#include "tsin.h"
 
 GtkWidget *gwin1;
 static GtkWidget *frame;
-static char *wselkey;
-static int wselkeyN;
+char *wselkey;
+int wselkeyN;
 //Window xwin1;
 
 #define SELEN (15)
-
+int win1_rowN;
 static GtkWidget *labels_sele[SELEN], *labels_seleR[SELEN];
 static GtkWidget *eve_sele[SELEN], *eve_seleR[SELEN];
 static GtkWidget *arrow_up, *arrow_down;
@@ -25,15 +26,13 @@ static int current_config()
     pho_candicate_R2L;
 }
 
-static int idx_to_x(int tN, int i)
+static int idx_to_y(int i)
 {
-    if (tN > pho_candicate_col_N)
-      tN = pho_candicate_col_N;
-
-    int x = i % tN;
-    if (pho_candicate_R2L)
-      x = tN - 1 - x;
-    return x;
+	return i % win1_rowN;
+}
+static int idx_to_x(int i)
+{
+	return i / win1_rowN;
 }
 
 static gboolean button_scroll_event_tsin(GtkWidget *widget,GdkEventScroll *event, gpointer user_data)
@@ -61,23 +60,7 @@ void create_win1()
   if (gwin1)
     return;
 
-  gwin1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_has_resize_grip(GTK_WINDOW(gwin1), FALSE);
-
-#if UNIX
-  gtk_window_set_resizable(GTK_WINDOW(gwin1), FALSE);
-#endif
-
-#if WIN32
-  set_no_focus(gwin1);
-#endif
-  gtk_widget_realize (gwin1);
-
-#if UNIX
-  set_no_focus(gwin1);
-#else
-  win32_init_win(gwin1);
-#endif
+  gwin1 = create_no_focus_win();
 
   g_signal_connect (G_OBJECT (gwin1), "scroll-event", G_CALLBACK (button_scroll_event_tsin), NULL);
 }
@@ -94,9 +77,13 @@ static void mouse_button_callback( GtkWidget *widget,GdkEventButton *event, gpoi
   switch (event->button) {
     case 1:
       v = GPOINTER_TO_INT(data);
+#if 1
       if (cb_sele_by_idx)
         cb_sele_by_idx(v);
       force_preedit_shift();
+#else
+      send_fake_key_eve(pho_selkey[v]);
+#endif
       break;
   }
 }
@@ -126,6 +113,8 @@ void create_win1_gui()
     return;
 //  dbg("create_win1_gui %s\n", wselkey);
 
+  win1_rowN = (wselkeyN + pho_candicate_col_N - 1 ) / pho_candicate_col_N;
+
   frame = gtk_frame_new(NULL);
   gtk_container_add (GTK_CONTAINER(gwin1), frame);
 
@@ -154,8 +143,9 @@ void create_win1_gui()
   int i;
   for(i=0; i < wselkeyN; i++)
   {
-    int y = i/pho_candicate_col_N;
-    int x = idx_to_x(SELEN+1, i);
+	int y = idx_to_y(i);
+	int x = idx_to_x(i);
+	dbg("i:%d y:%d x:%d\n", i, y, x);
 
     if (!tsin_tail_select_key)
       x*=2;
@@ -243,7 +233,7 @@ void clear_sele()
 
 char *htmlspecialchars(char *s, char out[]);
 
-void set_sele_text(int tN, int i, char *text, int len)
+void set_sele_text(int i, char *text, int len)
 {
   if (len < 0)
     len=strlen(text);
@@ -268,7 +258,7 @@ void set_sele_text(int tN, int i, char *text, int len)
 
   sprintf(selma, "<span %s>%s</span>", ul, htmlspecialchars(cc, uu));
 
-  int x = idx_to_x(tN, i);
+  int x = idx_to_x(i);
   char *sep= x?" ":"";
 
   if (tsin_tail_select_key) {
@@ -306,12 +296,13 @@ gboolean timeout_minimize_win1(gpointer data)
 
 #if WIN32
 #include <gdk/gdkwin32.h>
+extern int dpy_x_ofs, dpy_y_ofs;
 void set_win_pos_size(GtkWidget *win, int x, int y, int xl, int yl)
 {
   dbg("set_win_size %d,%d %d %d\n", x, y, xl, yl);
   HWND hwnd=(HWND)gdk_win32_drawable_get_handle(win->window);
 #if 1
-  SetWindowPos(hwnd, HWND_TOP, x, y, xl, yl, SWP_SHOWWINDOW);
+  SetWindowPos(hwnd, HWND_TOP, x + dpy_x_ofs, y + dpy_y_ofs, xl, yl, SWP_SHOWWINDOW);
 #else
   MoveWindow(hwnd, x, y, xl, yl, false);
 #endif
@@ -344,8 +335,15 @@ void disp_selections(int x, int y)
   if (x + win1_xl > dpy_xl)
     x = dpy_xl - win1_xl;
 
-  if (y + win1_yl > dpy_yl)
+  if (y + win1_yl > dpy_yl) {
     y = win_y - win1_yl;
+	// so that it will not cover AP's text. dirty fix, need the height of text
+	if (gcin_edit_display==GCIN_EDIT_DISPLAY_ON_THE_SPOT)
+	  y -= 24;
+  }
+
+  if (y < 0)
+    y = 0;
 
   gtk_window_move(GTK_WINDOW(gwin1), x, y);
 
@@ -464,7 +462,7 @@ void recreate_win1_if_nessary()
 }
 
 
-void set_wselkey(char *s)
+static void set_wselkey_str(char *s)
 {
   if (!wselkey || strcmp(wselkey, s)) {
     if (wselkey)
@@ -475,3 +473,36 @@ void set_wselkey(char *s)
 //    dbg("set_wselkey %s\n", s);
   }
 }
+
+#include "gtab.h"
+
+char *ch_mode_selkey(gboolean is_gtab)
+{
+  char *s;
+  if (is_gtab && cur_inmd)
+    s = cur_inmd->selkey;
+  else
+    s = pho_selkey;
+
+  return s;
+}
+
+char *en_sel_keys(gboolean is_gtab);
+void gtab_set_win1_cb(), tsin_set_win1_cb();
+
+void set_wselkey()
+{
+  gboolean is_gtab = current_method_type()==method_type_GTAB;
+  char *s;
+  if (tsin_pho_mode()) {
+	s = ch_mode_selkey(is_gtab);
+  } else
+    s =  en_sel_keys(is_gtab);
+
+  set_wselkey_str(s);
+
+  if (is_gtab)
+    gtab_set_win1_cb();
+  else
+	tsin_set_win1_cb();
+}
